package dbutils;
import java.util.*;
import java.io.*;


/** *************************************************************************************
<p>   
Abbildung einer Datenbanktabelle in eine Java-String-Struktur. Die Namen der Spalten 
werden bernommen. 
</p>  
<p>   
Sprach- und datenbankabhngige Konvertierungen sollten
beim Fllen der Tabelle erfolgen.
</p>
<p>
Diese Klasse ist entstanden, um innerhalb von Web-Anwendungen Tabellen auszugeben.
Dabei sollte in der Anwendungslogik mglichst wenig Abhngigkeit zu einer bestimmten
DB enthalten sein (insbesondere konnte nicht davon ausgagangen werden, das ein JDBC-Treiber
vorhanden ist). Weiterhin sollte auch niemand in die Versuchung kommen, Tabellenseiten
mit geffnetem DB-Cursor im Web zum Bltteren anzubieten. 
</p>

<h3> Vorgehen </h3> 
<samp>

<pre>

ERSTELLEN:
Datenbankabhngig wird die RamTable konstruiert, etwa in einer Klasse,
Db, welche eine bestimmte Datenbank abstrahiert (Oracle, Sas, ...).

RamTable rt = Db.getRamTable("(Select Name, Vorname from tabelle1)");
LESEN: 
Iterator   e = rt.iterator();
int  idxName = rt.getColumnIndex( "Name");
while (e.hasNext())
{
	aktuellerSatz = (String[])e.next()      ;
	String aktName = aktuellerSatz[ idxName ];
	...
	
}

</pre>
</samp>
*
*  @author   P. Seher
*  @version  1.0
*
*/


// ==================================================
// Klasse RamTable
// ==================================================

public class RamTable implements Serializable  {
	
	private String TableName;
	
	private String [] ColumnNames;
	private String [] ColumnLabels;
	
	private ArrayList DataList;
	
	public RamTable (String Table) {
		
		DataList= new ArrayList();
		TableName=Table;
		
		
	} // End Method
	// *********************** addColumnNames    *******************************	
	/**	
	
	Eine Zeile hinzufgen
	
	*/   			
	public boolean addRow (String[] row) {
		return DataList.add(row);
		
	} // End Method
	
	// *********************** addColumnNames    *******************************	
	/**	
	
	Die Spaltennamen in einem String-Array hinzufgen
	
	*/   		
	public void addColumnNames (String[] names) {
		ColumnNames=names;
	} // End Method
	
	
	// *********************** getColumnNames    *******************************	
	/**	
	
	Gibt die Spaltennamen als Stringarray
	
	*/   			
	
	public String [] getColumnNames() {
		return ColumnNames;
	}  // End Method
	
	// *********************** getColumnIndex*******************************	
	/**	
	
	Gibt den Index der Spalte mit dem Namen 'colName' zurck.
	Die erste Spalte hat den Index 0
	
	*/   		
	public int getColumnIndex( String colName) {
		int idx=-1;
		for (int i = 0; i < ColumnNames.length; i++) {
			if (ColumnNames[i].equals(colName)) { idx=i; break;}
		}
		return idx;
	}  // End Method
	
	// *********************** iterator    *******************************	
	/**	
	
	Gibt einen Iterator ueber die Zeilenliste zurck.
	
	*/   		
	public Iterator iterator() {
		return DataList.iterator();
	}  // End Method
	
	
	// *********************** isEmpty    **********************************	
	/**	
	
	Prft, ob die Tabelle keine Zeilen hat
	
	*/   		
	public boolean isEmpty() {
		return DataList.isEmpty();
	}  // End Method
	
	// *********************** size    *************************************	
	/**	
	
	Gibt die Anzahl der Zeilen zurck
	
	*/   	
	public int size() {
		return DataList.size();
	}  // End Method
	
	// *********************** getCell    *****************************************	
	/**	
	
	Gibt den Inhalt der Zelle in Zeile 'row' und Spalte 'col' zurck;
	(0,0) sind die Koordinaten der ersten Zelle.
	
	*/   		
	public String  getCell(  int row, int col) {
		String colValue;
		try {
			colValue=(getRow(row))[col];
		}
		catch (Exception ex) { colValue=null;}
		
		return colValue;
	}  // End Method
	
	// *********************** getRow    *****************************************	
	/**	
	
	Gibt die Zeile 'row' zurck;
	
	*/   	
	
	public String [] getRow(  int index) {
		return ( String []) DataList.get(index);
	}  // End Method
	
	// *********************** getRow(  String colName, String colValue)  ********* 
	/**
	<p>
	Gib die erste Zeile der Tabelle, bei der in einer gegebenen Spalte
	ein bestimmter Wert enthalten ist. Fhrt dieser Filter zu keinem Treffer,
	ist das Ergebnis <samp> null </samp>.
	</p>
	
	<p>
        In SQL-Notation:
	</p>
	<samp>
	<pre>
	select * 
	from table 
	where whereColName=whereColValue  
        </pre>
	</samp>
	
	@param  colName   Name derSpalte
	@param  colValue  Wert der Spalte
	
	*/	
	public String [] getRow(  String colName, String colValue) {
		
		String [] gefundenerSatz =null;
		String [] aktuellerSatz =null;
		int colIdx = -1;
		
		// -- Spaltenindex bestimmen
		colIdx = getColumnIndex( colName);
		if (colIdx < 0) return null;
		
		// -- Satz suchen
		Iterator e=DataList.iterator();
		while (e.hasNext())
                {
			aktuellerSatz = (String[])e.next()      ;
			if (aktuellerSatz[colIdx].equals(colValue) ) {
				gefundenerSatz =aktuellerSatz;
				break;
			}
		}
		return gefundenerSatz;
	}  // End Method
	// *********************** getFirstCol(  String colName, String colValue)  ********* 
	/**
	
	Gib den <b> Inhalt einer Zelle </b> mit der Spalte colName
	und der Zeileneinschrnkung (Filter)  whereColName und whereColValue.
	Wenn der Filter zu keinem Treffer fhrt, ist das Ergebnis <samp> null </samp>.
	
	<p>
        In SQL-Notation:
	</p>
	<samp>
	<pre>
	select colName 
	from table 
	where whereColName=whereColValue  
        </pre>
	</samp>
	<p>
	<b>wobei nur die "erstbeste" Zeile bercksichtigt wird</b>.
	Eindeutig ist der Treffer, wenn der Filter auf eine Spalte gesetzt wird, 
	die Primrschlssel in der  Ursprungstabelle ist. 
	
	</p>     
	
	
	@param  colName       Name derSpalte, deren Inhalt interessiert.
	@param  whereColName  Name der Spalte, fuer die eine Einschrnkung gemacht
	werden soll
	@param  whereColValue Wert der Spalteneinschrnkung 		      
	
	*/		
	public String getFirstCell(  String whereColName, String whereColValue, String colName) {
		
		String  gefundeneSpalte =null;
		String [] gefundenerSatz =getRow(   whereColName,  whereColValue);
		int colIdx = getColumnIndex( colName);
		if ( gefundenerSatz != null && colIdx >= 0 ) gefundeneSpalte= gefundenerSatz[colIdx];
		
		return gefundeneSpalte;
	}  // End Method
	
	
	public SortedSet getColumnSet( String colName )
	{
		return getColumnSet( getColumnIndex( colName) );		
	}
	
	public SortedSet getColumnSet( int columnIndex )
	{
		TreeSet       colSet  = new TreeSet();
		
		Iterator e=DataList.iterator();
		while (e.hasNext())
                {
			String [] aktuellerSatz = (String[])e.next();
			colSet.add(aktuellerSatz[columnIndex]);
		}
		
		return colSet;
		
	}
	
} // End class
