package excel;

import dbutils.TableReader;
import org.apache.poi.poifs.filesystem.POIFSFileSystem;
import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.hssf.util.HSSFColor;
import org.apache.poi.hssf.util.RangeAddress;

import java.io.*;
import java.util.*;
import java.text.*;

/** *************************************************************************************
*         Klasse Sheet
*  @author   P. Seher
*  @version  0.1 3.2004  
*
*/


public class Sheet {
	
	//    ---Variable fuers Package
	
  	HSSFWorkbook     cls_wb;
	HSSFSheet        cls_sheet;  // eigentliches POI-Sheet;
        // Styles 
	HSSFCellStyle   cls_styleHeader ;   
	HSSFCellStyle   cls_styleRow ;
	HSSFCellStyle   cls_styleRowHeader;
	HSSFCellStyle   cls_styleRowHeaderNum ;
	HSSFCellStyle   cls_styleKeyFigure ;
	
	//    --- private Variable  
	private  String   cls_sheetName;
	private  String   cls_Header;
	private  String   cls_RangeName;
	
	// *********************** Konstruktor    *************************************************************	
	/**
	
	
	@param  sheetName   Sheet Name
	
	*/
	
	Sheet(String sheetName)
	{
		cls_sheetName  = sheetName; 		
	}
	
	// *********************** setHeader    *************************************************************	
	/**
	
	
	@param  header  Ueberschrift Kopfzeile
	
	*/
	
	public void setHeader ( String header)
	{
		cls_Header = header;	
	}
	
	// *********************** setRangeName    *************************************************************	
	/**
	
	
	@param  rangeName  Excel-Range Name, indem der Bereich fuer die 
	erzeugten Daten hinterlegt wird
	
	*/
	
	public void setRangeName ( String rangeName)
	{
		cls_RangeName = rangeName;	
	}
	
	// *********************** createRow    *************************************************************	
	/**
	
	
	@param  rangeName  Excel-Range Name, indem der Bereich fuer die 
	erzeugten Daten hinterlegt wird
	
	*/
	
	public boolean createRow ( short rownumExcel, Object[] cellValues )	
	{
		boolean        result  =  true;
		HSSFRow        row     =  null;
		HSSFCell       cell    =  null;
		
		try {
			
			row = cls_sheet.getRow(rownumExcel);
			if (row == null)
			{
				// Zeile noch nicht vorhanden
			        row = cls_sheet.createRow(rownumExcel);
			}
			else
			{
				// Zeile bereits vorhanden
				// Sie bleibt erhalten, aber alle Zellen werden gelscht
				
				java.util.Iterator cellIterator= row.cellIterator();
				while (cellIterator.hasNext()) 
				{
					cellIterator.next();
					cellIterator.remove();
				}
				
				
			}
			
			for ( short j=0; j< cellValues.length; j++)
			{
				// System.out.println("Zeile:"+rownumExcel+" Spalte:"+j+" Wert:"+cellValues[j].toString());
				cell        = row.createCell(j);
				cell.setCellValue(cellValues[j].toString());
				// cell.setCellStyle(cls_styleRowHeader);
				
			}
		    } 
		catch (Exception e) 
		{
			e.printStackTrace(System.err); 
			System.out.println("<<< Fehler beim Erstellen der Zeile '"+rownumExcel+"' >>>");
			result=false;
		}
		
		return result;
	}

	
	// *********************** import    *************************************************************	
	/**
	
	
	@param  rt  Zu importierende Tabelle
	
	*/	
	public	boolean importTable (TableReader rt)
	{
		boolean result=true;
		int   anzahlSpalten= rt.getColumnCount();
		//String[] spalten=rt.getColumnNames();
		int  [] spaltenBreite=new int[anzahlSpalten]; // es wird hier die optimale Spaltenbreite, auf Grund des Inhaltes
		// der Zellen und der berschrift ermittelt
		
		short firstRowExcel = 0;
		short firstColExcel = 0;
		short lastColExcel  = (short) (firstColExcel + anzahlSpalten - 1); 
		short lastRowExcel  = -1;             // mit Tabellenkopfzeile
	        
		
		short rownumExcel   =  firstRowExcel;  // logische HSSF-Anfangszeile
		short colnumExcel   =  firstColExcel;  // logische HSSF-Anfangsspalte
		
		
		try {
			
			HSSFRow        row    =  null;
			HSSFCell       cell   =  null;
			
			
			row         = cls_sheet.createRow(rownumExcel);
			colnumExcel = firstColExcel;
			
			
			// ---------------------------------------------------------------------
			// Tabellen-Ueberschrift generieren
			// ---------------------------------------------------------------------
			
			
			for (short i=0;i<anzahlSpalten; i++,colnumExcel++)
			{		
				String label= rt.getColumnLabel(i+1);
				cell        = row.createCell(colnumExcel);
				cell.setCellValue(label);
				cell.setCellStyle(cls_styleHeader);
				spaltenBreite[i]=label.length();
				
			}	
			
			
			// ---------------------------------------------------------------------
			// Tabellen-Zeilen generieren
			// ---------------------------------------------------------------------
			
			
			while (rt.next())	
			{
				rownumExcel++;
				colnumExcel=firstColExcel;
				
				row=cls_sheet.createRow(rownumExcel);
				
				for ( short j=0; j< anzahlSpalten; j++)
				{
					cell=row.createCell(colnumExcel++);
					
					if (rt.isKeyFigure (j+1))
					{					
						cell.setCellValue(rt.getDouble(j+1));
						cell.setCellStyle(cls_styleKeyFigure);
					}
					else 
					{
						if (rt.isNumber(j+1))
						{
							cell.setCellValue( rt.getDouble(j+1) );
							cell.setCellStyle(cls_styleRowHeaderNum);						 
						}
						else
						{
							cell.setCellValue( rt.getString(j+1) );
							cell.setCellStyle(cls_styleRowHeader);
						}
					}
					
					// Maximale Spaltenlnge bestimmen
					spaltenBreite[j]=Math.max(spaltenBreite[j],rt.getCellDisplaySize(j+1));
				}	
				
			}
			
			// ---------------------------------------------------------------------
			// Datenbereich als EXCEL-Namensobjekt erzeugen
			// ---------------------------------------------------------------------
			
			lastRowExcel  = (short)(rownumExcel); 
			
			String rangeName = cls_RangeName;
			if ( rangeName == null ) rangeName = "RANGE_"+cls_sheetName;
			else
			{
				rangeName=rangeName.trim();
				if (rangeName.length()==0) rangeName = "RANGE_"+cls_sheetName;
	                }
			
			HSSFName excelName=cls_wb.createName();
			excelName.setNameName(rangeName);
			// ---  Syntax:    RangeAddress (startCol,startRow, endCol, endRow);
			
			
			RangeAddress range = new RangeAddress( (short)firstColExcel+1, 
			(short)firstRowExcel+1,
			(short)lastColExcel+1,
			(short)lastRowExcel+1
			);
			//System.out.println("firstColExcel:"+firstColExcel+",firstRowExcel:"+firstRowExcel+",lastColExcel:"+lastColExcel+",lastRowExcel:"+lastRowExcel);
			//System.out.println( "Adresse:"+range.getAddress() );
			// --- Hier ist das Ergebnis ein RELATIVER Bezug, etwa "A2:E100"; 
			
			//excelName.setReference( pSheet+"!"+range.getAddress() );
			// --- Bei Pivottabellen ist aber ein ASOLUTER bezug notwendig, etwa "$A$2:$E$100
			excelName.setReference(cls_sheetName+"!"+
			"$"+range.numTo26Sys(firstColExcel+1) +
			"$"+(firstRowExcel+1)+
			":"+
			"$"+range.numTo26Sys(lastColExcel+1)+
			"$"+(lastRowExcel+1)
			);
			
			
			//System.out.println("EXCEL-Name:"+excelName.getNameName()+"="+excelName.getReference());
			
			
			// ---------------------------------------------------------------------
			// Optimale-Spaltenbreite generieren
			// ---------------------------------------------------------------------
			
			colnumExcel = firstColExcel;
			
			for (short i=0;i<spaltenBreite.length; i++,colnumExcel++)
			{
				int breite=spaltenBreite[i]+4;
				//System.out.println( "Spalte"+rt.getColumnLabel(i+1)+" Breite:"+breite);
				cls_sheet.setColumnWidth(colnumExcel,(short) (256*breite));
			}
			// ---------------------------------------------------------------------
			// Kopf- und Fusszeile
			// ---------------------------------------------------------------------	
			HSSFHeader     header = cls_sheet.getHeader();
			header.setCenter(header.fontSize((short) 16)+cls_sheetName);
			
			HSSFFooter     footer = cls_sheet.getFooter();
			//footer.setRight("Seite "+footer.page()+" von "+ footer.numPages());
			// so erst mal "sprachunabhaengig" 
                        footer.setRight(footer.page()+"-"+ footer.numPages());				
			
			
			
			
		} catch (Exception e) 
		{
			e.printStackTrace(System.err); 
			System.out.println("<<< Fehler beim Importieren  >>>");
			result=false;
		}
		
		
		return result;
	}
	
	
}
