package excel;

import org.apache.poi.poifs.filesystem.POIFSFileSystem;
import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.hssf.util.HSSFColor;
import org.apache.poi.hssf.util.RangeAddress;

import java.io.*;
import java.util.*;
import java.text.*;

/** *************************************************************************************
*         Klasse Workbook
*  @author   P. Seher
*  @version  0.1 3.2004  
*
*/
 

  public class Workbook {
	  
	  
	 private       HSSFWorkbook     cls_wb ;       // Workbook-Objekt
	 private       String           cls_fileName;  // Name der Ausgabedatei
	 
	 protected     HSSFPalette      cls_palette;   // Farbpalette
	 
	 // Styles 
	 protected	HSSFCellStyle   cls_styleHeader ;   
	 protected	HSSFCellStyle   cls_styleRow ;
	 protected	HSSFCellStyle   cls_styleRowHeader;
	 protected	HSSFCellStyle   cls_styleRowHeaderNum ;
	 protected	HSSFCellStyle   cls_styleKeyFigure ;
	 
    // ***************************  Konstruktor ********************************
     /**
     */	 
	 
     public Workbook() 
     {
	     cls_wb         =  null;
	     cls_fileName   =  null;
	     
     }
    // ***************************  open   *************************************
     /**
      *   Excel-Workbook mit Vorlage ffnen
      *
      *         @param  pFileName   Name der Excel-Ausgabedatei
      *         @param  pVorlage    Name der Excel-Vorlagedatei
     */
     public boolean open( String pFileName, String pVorlage) throws java.io.FileNotFoundException, 
                                                                    java.io.IOException
     {
	     boolean result=false;
		     	  
	     POIFSFileSystem    fs   = new POIFSFileSystem (new FileInputStream(pVorlage));
	     cls_wb       = new HSSFWorkbook(fs);
             cls_fileName = pFileName; 
             initWorkbook();
             result=true;   			  
	     
	     return result;
     }
     // ***************************  open   ************************************************************
     /**
      *   Excel-Workbook OHNE Vorlage ffnen
      * 
      *         @param  pFileName   Name der Excel-Ausgabedatei
     */ 
     
  public boolean open( String pFileName )
  {
	  cls_fileName =  pFileName;
	  cls_wb       =  new HSSFWorkbook();
	  initWorkbook();
	  return true;
  }

  
  // ***************************  open   ************************************************************
  /**
  *   Excel-Workbook  als Excel-Datei schreiben 
  * 
  */ 
  
  public boolean write() throws java.io.IOException
  {
	  boolean result=false;
	  
	  // Write the output to a file
	  FileOutputStream fileOut = new FileOutputStream( cls_fileName);
	  cls_wb.write(fileOut);
	  fileOut.close();
	  result=true;
	  
	  
	  
	  return result;
	  
	  
  }
  
     // ***************************  open   ************************************ 
     /**
      *   Excel-Workbook  schliessen
      * 
     */ 
     
     public void close()
     {
	return;     
     }
     
     
     // ***************************  createSheet   **************************** 
     /**
      *   Excel-Sheet erzeugen
      
      	@param  sheetName    Name des Excel-Sheets, das erzeugtwerden soll;
	                     ein bereits vorhandenes Sheet mit gleichem Namen
			     wird berschrieben
      * 
     */ 
     
     public Sheet createSheet(String sheetName)
     {
	     Sheet sheet = new Sheet(sheetName);
	     
	     sheet.cls_wb    = cls_wb;
	     sheet.cls_sheet = cls_wb.createSheet(sheetName);
	     
	     sheet.cls_styleHeader        = cls_styleHeader;
	     sheet.cls_styleRow           = cls_styleRow;
	     sheet.cls_styleRowHeader     = cls_styleRowHeader;
	     sheet.cls_styleRowHeaderNum  = cls_styleRowHeaderNum; 
	     sheet.cls_styleKeyFigure     = cls_styleKeyFigure;
	     
	      
	     return sheet;     
     } 
     
     // ***************************  getSheet   **************************** 
     /**
      *   Gib eine Referenz zu einem Excel-Sheet.
      
      	@param  sheetName    Name des Excel-Sheets.
	                     Ist das Sheet mit diesem Namen nicht vorhanden,
			     ist das Ergebnis die Referenz 'null'.
      * 
     */  
     
     public Sheet getSheet (String sheetName)
     {
	     
	     Sheet sheet = null;
	     if ( existsSheet(sheetName))
	     {
		     sheet = new Sheet(sheetName);
		     sheet.cls_wb    = cls_wb;
		     sheet.cls_sheet = cls_wb.getSheet(sheetName);		     
	     }
	     
	     
	     return sheet;     
     }     
     
     
     
     
     
 	// *********************** createNamedRange()    ********************************************************* 
	/**
	Es wird ein Excel-Name erzeugt,  der sich auf einen Excel-Bereich bezieht. Die Bereichsangaben werden in absolute
	Excel-Adressen umgesetzt.
	
	Falls der angegebene 'rangeName' schon existiert, wird er berschrieben. 
	
	
	@param  rangeName  Excel-Range Name, indem der Bereich fuer die 
	                   erzeugten Daten hinterlegt wird
	@param  sheetName  Excel-Sheet aus das sich der zu definierende Name bezieht
	
	@param  range      Bezugsbereich innerhalb des Excel-Sheets
	
	                   range[0]: firstColExcel
			   range[1]: firstRowExcel
			   range[2]: lastColExcel
			   range[3]: lastRowExcel			   

	*/
	
	public boolean createNamedRange ( String rangeName, 
	String sheetName, 
	short firstColExcel,
	short firstRowExcel,
	short lastColExcel,
	short lastRowExcel 	
 )	
	{
		boolean        result  =  true;
		
		
		try {
			
			HSSFName excelName=cls_wb.createName();
			if ( existsName(rangeName) ) removeName(rangeName);
			
			excelName.setNameName(rangeName);
			// ---  Syntax:    RangeAddress (startCol,startRow, endCol, endRow);
			
			
			RangeAddress range = new RangeAddress( (short)firstColExcel+1, 
			(short)firstRowExcel+1,
			(short)lastColExcel+1,
			(short)lastRowExcel+1
			);
			//System.out.println("firstColExcel:"+firstColExcel+",firstRowExcel:"+firstRowExcel+",lastColExcel:"+lastColExcel+",lastRowExcel:"+lastRowExcel);
			//System.out.println( "Adresse:"+range.getAddress() );
			// --- Hier ist das Ergebnis ein RELATIVER Bezug, etwa "A2:E100"; 
			
			//excelName.setReference( pSheet+"!"+range.getAddress() );
			// --- Bei Pivottabellen ist aber ein ASOLUTER bezug notwendig, etwa "$A$2:$E$100
			excelName.setReference(sheetName+"!"+
			"$"+range.numTo26Sys(firstColExcel+1) +
			"$"+(firstRowExcel+1)+
			":"+
			"$"+range.numTo26Sys(lastColExcel+1)+
			"$"+(lastRowExcel+1)
			);
			
			
			System.out.println("EXCEL-Name:"+excelName.getNameName()+"="+excelName.getReference());
			
			
		} 
		catch (Exception e) 
		{
			e.printStackTrace(System.err); 
			result=false;
		}
		
		return result;
	}
	
  	// *********************** getNumberOfNames()    ********************************************************* 
	/**

	*/
	public int getNumberOfNames ()
	{
		return cls_wb.getNumberOfNames();
	}
	
	// *********************** getNumberOfSheets()    ********************************************************* 
	/**

	*/
	public int getNumberOfSheets ()
	{
		return cls_wb.getNumberOfSheets();
	}
	
	// *********************** removeName()    ********************************************************* 
	/**
	
	@param  name    Excel-Name, der gelscht werden soll;

	*/
	public void removeName ( String name)
	{
		cls_wb.removeName (name);		
	}
	
	// *********************** removeSheet()    ********************************************************* 
	/**
	
	@param  sheetName    Name des Excel-Sheets, das gelscht werden soll;

	*/	
	public void removeSheet ( String sheetName)
	{
		int anz = getNumberOfSheets ();
		
		for (int i=0; i<anz; i++)
		{
			String aktSheetName = cls_wb.getSheetName(i);
			if ( aktSheetName.equals(sheetName) )
			{
				cls_wb.removeSheetAt(i);
				return;
			}
		}
		return;
				 		
	}
	
	// *********************** existsSheet()    ********************************************************* 
	/**
	
	@param  sheetName    Name des Excel-Sheets, dessen Existenz geprft werden soll.

	*/	
	public boolean existsSheet ( String sheetName)
	{
		
		int anz = getNumberOfSheets ();
		
		for (int i=0; i<anz; i++)
		{
			String aktSheetName = cls_wb.getSheetName(i);
			if ( aktSheetName.equals(sheetName) ) return true;

		}
		return false;
				 		
	}
	// *********************** existsSheet()    ********************************************************* 
	/**
	
	@param  name    Name des Excel-Namensobjektes, dessen Existenz geprft werden soll.

	*/	
	public boolean existsName ( String name)
	{
		
		int anz = getNumberOfNames ();
		
		for (int i=0; i<anz; i++)
		{
			String aktName = cls_wb.getNameName(i);
			if ( aktName.equals(name) ) return true;

		}
		return false;
				 		
	}		
		
	
	
    
  /*
  ================================================================================================================ 
                            PROTECTED AREA
  ================================================================================================================ 
  */
 
  protected void setPalette()
  {
		// ---------------------------------------------------------------------
		//Farbpalette bestimmen
		// ---------------------------------------------------------------------		
		cls_palette=cls_wb.getCustomPalette();
		//-- "Ocker etwas gedeckt"
		cls_palette.setColorAtIndex(HSSFColor.GOLD.index, (byte) 230, (byte) 200, (byte) 150);
		//-- Dunkles Bordeauxrot
		cls_palette.setColorAtIndex(HSSFColor.RED.index, (byte) 170, (byte) 50, (byte) 50);
		//-- Dunkelgrau
		// so im Styleguide: palette.setColorAtIndex(HSSFColor.GREY_50_PERCENT.index, (byte) 130, (byte) 130, (byte) 130);
		cls_palette.setColorAtIndex(HSSFColor.GREY_50_PERCENT.index, (byte) 100, (byte) 100, (byte) 100);
		//-- Sandfarben
		cls_palette.setColorAtIndex(HSSFColor.LIGHT_YELLOW.index, (byte) 236, (byte) 225, (byte) 200);	  
  	  
  }
  
   
  protected void setStyles()
  {		
	       // ---------------------------------------------------------------------
		// Styles festlegen
		// ---------------------------------------------------------------------
		HSSFCellStyle styleHeader         =  cls_wb.createCellStyle();
		HSSFCellStyle styleRow            =  cls_wb.createCellStyle();
		HSSFCellStyle styleRowHeader      =  cls_wb.createCellStyle();
		HSSFCellStyle styleRowHeaderNum   =  cls_wb.createCellStyle();
		HSSFCellStyle styleKeyFigure      =  cls_wb.createCellStyle();
		
		HSSFFont      fontHeader          =  cls_wb.createFont();
		HSSFFont      fontRow             =  cls_wb.createFont();
		
		HSSFDataFormat format             =  cls_wb.createDataFormat();
		
		// --- "Fonts"
		fontHeader.setFontName(fontHeader.FONT_ARIAL);
		fontHeader.setBoldweight(fontHeader.BOLDWEIGHT_BOLD);
		fontHeader.setColor(HSSFColor.GREY_50_PERCENT.index);
		
		fontHeader.setFontName(fontHeader.FONT_ARIAL);
		fontHeader.setBoldweight(fontHeader.BOLDWEIGHT_BOLD);
		fontHeader.setColor(HSSFColor.GREY_50_PERCENT.index);
		
		// --- Tabellen-Kopf
		styleHeader.setFillPattern(HSSFCellStyle.SOLID_FOREGROUND);
		styleHeader.setFillForegroundColor(HSSFColor.GOLD.index);
		styleHeader.setBorderBottom(styleHeader.BORDER_THICK);
		styleHeader.setBorderTop(styleHeader.BORDER_THICK);
		styleHeader.setBorderLeft(styleHeader.BORDER_THIN);
                styleHeader.setBorderRight(styleHeader.BORDER_THIN);
		
		
		styleHeader.setRightBorderColor(HSSFColor.GREY_50_PERCENT.index);
		styleHeader.setLeftBorderColor(HSSFColor.GREY_50_PERCENT.index);
		styleHeader.setTopBorderColor(HSSFColor.GREY_50_PERCENT.index);
		styleHeader.setBottomBorderColor(HSSFColor.GREY_50_PERCENT.index);
		
		styleHeader.setAlignment(styleHeader.ALIGN_CENTER);	
		styleHeader.setFont(fontHeader);
		
		// --- "Normale" Zeilen 
		
		styleRow.setAlignment(styleRow.ALIGN_CENTER);
		styleRow.setBorderBottom(styleRow.BORDER_THIN);
		
		// --- Zeilen-Kopf als String
		styleRowHeader.setFillPattern(HSSFCellStyle.SOLID_FOREGROUND);
		styleRowHeader.setFillForegroundColor(HSSFColor.LIGHT_YELLOW.index);
		styleRowHeader.setBorderBottom(styleRowHeader.BORDER_THIN);
		styleRowHeader.setBorderTop(styleRowHeader.BORDER_THIN);
                styleRowHeader.setBorderLeft(styleRowHeader.BORDER_THIN);
                styleRowHeader.setBorderRight(styleRowHeader.BORDER_THIN);
		
		styleRowHeader.setRightBorderColor(HSSFColor.GREY_50_PERCENT.index);
		styleRowHeader.setLeftBorderColor(HSSFColor.GREY_50_PERCENT.index);
		styleRowHeader.setTopBorderColor(HSSFColor.GREY_50_PERCENT.index);
		styleRowHeader.setBottomBorderColor(HSSFColor.GREY_50_PERCENT.index);
		
		styleRowHeader.setAlignment(styleRowHeader.ALIGN_LEFT);				
		styleRowHeader.setFont(fontHeader);
		
		// --- Zeilen-Kopf als Zahl z.B. Monat
		styleRowHeaderNum.setFillPattern(HSSFCellStyle.SOLID_FOREGROUND);
		styleRowHeaderNum.setFillForegroundColor(HSSFColor.LIGHT_YELLOW.index);
		styleRowHeaderNum.setBorderBottom(styleRowHeaderNum.BORDER_THIN);
		styleRowHeaderNum.setBorderTop(styleRowHeaderNum.BORDER_THIN);
                styleRowHeaderNum.setBorderLeft(styleRowHeaderNum.BORDER_THIN);
                styleRowHeaderNum.setBorderRight(styleRowHeaderNum.BORDER_THIN);
		
		styleRowHeaderNum.setRightBorderColor(HSSFColor.GREY_50_PERCENT.index);
		styleRowHeaderNum.setLeftBorderColor(HSSFColor.GREY_50_PERCENT.index);
		styleRowHeaderNum.setTopBorderColor(HSSFColor.GREY_50_PERCENT.index);
		styleRowHeaderNum.setBottomBorderColor(HSSFColor.GREY_50_PERCENT.index);
		
		styleRowHeaderNum.setAlignment(styleRowHeaderNum.ALIGN_RIGHT);				
		styleRowHeaderNum.setFont(fontHeader);
		styleRowHeaderNum.setDataFormat(format.getFormat("0"));
		
		// --- Kennzahlen 
		
		styleKeyFigure.setAlignment(styleKeyFigure.ALIGN_RIGHT);
		styleKeyFigure.setBorderBottom(styleKeyFigure.BORDER_THIN);
		//styleKeyFigure.setDataFormat(format.getFormat("#,##0.00"));
		styleKeyFigure.setDataFormat(format.getFormat("#,##0"));
		
		styleKeyFigure.setBorderBottom(styleKeyFigure.BORDER_THIN);
		styleKeyFigure.setBorderTop(styleKeyFigure.BORDER_THIN);
                styleKeyFigure.setBorderLeft(styleKeyFigure.BORDER_THIN);
                styleKeyFigure.setBorderRight(styleKeyFigure.BORDER_THIN);
		
		styleKeyFigure.setRightBorderColor(HSSFColor.GREY_50_PERCENT.index);
		styleKeyFigure.setLeftBorderColor(HSSFColor.GREY_50_PERCENT.index);
		styleKeyFigure.setTopBorderColor(HSSFColor.GREY_50_PERCENT.index);
		styleKeyFigure.setBottomBorderColor(HSSFColor.GREY_50_PERCENT.index);
		
		// --- Ergebnis in Klassenvariablen festhalten
		
		 cls_styleHeader         = styleHeader;
                 cls_styleRow            = styleRow;
                 cls_styleRowHeader      = styleRowHeader;
                 cls_styleRowHeaderNum   = styleRowHeaderNum;
                 cls_styleKeyFigure      = styleKeyFigure; 

  }
  
  
  
  
  /*
  ================================================================================================================ 
                            PRIVATE AREA
  ================================================================================================================ 
  */
  private void initWorkbook()
  {
	  setPalette();
	  setStyles();
	  
  }
  
  
  
  
  
  }
