package mobile;

//import sasdb.*;
import asciidb.*;
import dbutils.*;
import mobile.*;
import com.Ostermiller.util.*;

import java.util.*;
import java.text.*;
import java.net.*;
import java.lang.*;

import java.io.*;

/** *************************************************************************************
<p>   
Metainformationen fr ISIS-Mobile erstellen.
Dazu gehren:
</p>
<ul>

<li>Spaltenbeschreibung fr alle ASCII-Tabellen.</li>

<li>SAS-Exportprogramme passend zu obiger Spaltenbeschreibung.
    D.h. diejenigen Programmteile, die aus dem ISIS-Datamart  Daten 
    fr einen Vertreter extrahieren. Die Vertreterdaten werden als
    spaltenorientierte ASCII-Dateien zwecks Weiterverabeitung gespeichert.
</li>

<li>Auftragstabellen fr Monats- und Tageslufe. </li> 

<li>Information ber die Schemaerstellung, d.h. wann hat dieses Programm ein Schema-Objekt erstellt. </li>
</ul>   

<p>
Diese Metainformationen werden als serialisiertes JAVA-Objekt der Klasse mobile.Schema abgepeichert. Der Pfad dieser Datei
wird ber die Methode getPathSchema() der Klasse MGlobal verwaltet.
</p>
<p>
Basis fr die Metadatenerstellung sind relationale Tabellen:
</p>
<ul>
<li>"kva_ct.mct_excel"</li>
</ul> 

*
*  @author   P. Seher, modified by Cl. Kraus
*  @version  1.0 
*
*/
public class CreateMeta {
	
	static        mobile.Schema  cls_schema;
	static        mobile.MGlobal cls_global;
		
	
	static int FEHLER_INITIALISIERUNG   =  110;
	static int FEHLER_VERARBEITUNG      =  112;
	static int FEHLER_CSVFILE			      =  113;
	

	public static String SCHEMA_VERSION        =  "1.0";
	public static char CSVSEPARATOR		         =  ';';
	
	
	public static void main (String[] args)  
	{
		int ergebnis = 1; 
		CreateMeta m = new CreateMeta();
		cls_schema     = new mobile.Schema();
		cls_global     = new mobile.MGlobal();
			
		ergebnis= m.createFieldDesc();
				
		if ( 0 == ergebnis) 			
		{       System.out.println("-------------------------------------------------------------------------------");
			System.out.println("Es wurde generiert:\n");
			System.out.println("Metadaten als JAVA-Klasse unter \n\t"+cls_global.getPathSchema()+"\n ");
			System.out.println("\n --- Alles O.K.  --- \n\n");
		}
		else                System.out.println("\n\n --- Generierung fehlerhaft !!!.  --- \n\n");
		
		System.exit(ergebnis); 
	} // End main

	
	
	
	private int createFieldDesc()
	{
		int ergebnis = 0;
    RamTable 	mylt 	= null;
		RamTable 	myltm 	= null;
		RamTable 	myltd 	= null;

		try {

			// -----------------------------------------------------------------
			//  Auftragstabellen fr Monats- und Tageslufe erstellen
			// -----------------------------------------------------------------
			String csvDatei = cls_global.getPathMeta()+"/schema.csv";
			FileInputStream in = null;
			File f = null;
			
			try {
				f = new File( csvDatei );
				byte buffer[] = new byte[ (int) f.length() ];
				in = new FileInputStream( f );
				in.read( buffer, 0, (int) f.length() );

				String str = new String( buffer );

				in.close();

				// parse the data
				String[][] values = CSVParser.parse( new StringReader(str),	CSVSEPARATOR );
				
				// min. 1 (data) entry necessary
				if ( values.length <= 1) return FEHLER_CSVFILE;

				
				String [] colArray = null;
				mylt = new RamTable(values[1][0]);  // nimm ersten Eintrag nach der Ueberschrift
				myltm = new RamTable( "setOrderMonthly" );
				myltd = new RamTable( "setOrderDaily" );
				
				for (int row=0; row<values.length; row++) {
					
						colArray = new String[ values[row].length - 1 ];
						for (int col=0; col<values[row].length - 1; col++) {
							colArray[col] = values[row][col];
							System.out.print(colArray[col]+" ");
						}
						System.out.println("");
						if ( row==0 ) {
							// set column names
							mylt.addColumnNames( colArray );
							myltm.addColumnNames( colArray );
							myltd.addColumnNames( colArray );
						} else {
							// set column data
							mylt.addRow( colArray );
						}
				}
				System.out.println("");
				
				int idxId = mylt.getColumnIndex("ID");
				int idxView = mylt.getColumnIndex("DATEN_QUELLE");
				int idxLauf = mylt.getColumnIndex("LAUF");
				
				Iterator e = mylt.iterator();
				
				while ( e.hasNext() ) {
					
					String[] zeile = ( String[]) e.next();
					String view  =  zeile[ idxView ];
					String lauf  =  zeile[ idxLauf ];

					// -----------------------------------------------------------------
					//  Labels fr Merkmale und Kennzahlen
					// -----------------------------------------------------------------
					// Zugehrige Spaltenbeschreibung merken
					String [][] classAndLabels = readClassAndLabels( view );
					
					// Debug Ausgabe
					//for (int row=0; row<classAndLabels.length; row++) {
					//		for (int col=0; col<classAndLabels[row].length; col++) {
					//			System.out.print(classAndLabels[row][col]+" ");
					//		}
					//		System.out.println("");
					//}

					if ( lauf.equals("M") ) {
						System.out.println("setOrderMonthly");
						myltm.addRow( zeile );
					} else {
						System.out.println("setOrderDaily");
						myltd.addRow( zeile );
					}
				}
			

				
			} catch (Exception e) 
			{
				e.printStackTrace(System.err); 
				System.out.println("<<< csv-Datei '"+csvDatei+"' kann nicht gelesen werden :-( >>>");
				in.close();
			}

			cls_schema.setOrderMonthly( (RamTable)myltm );   // Auftrge MONAT
			cls_schema.setOrderDaily( (RamTable)myltd );     // Auftraege TAG

			// -----------------------------------------------------------------
			//  Information ber die Schemaerstellung
			// -----------------------------------------------------------------

			SortedMap   ltInfo  = new TreeMap();
			ltInfo.put ("DATUM",new java.util.Date());
			ltInfo.put  ("VERSION",SCHEMA_VERSION);
	                cls_schema.setInfoMap( ltInfo );   // Beschreibung der Excel-Generierung
			
			// -----------------------------------------------------------------
			//  Schema als Datei schreiben
			// -----------------------------------------------------------------
			
			MyIo.writeObject(cls_schema, cls_global.getPathSchema());
			
		} catch (Exception e) 
		{
			e.printStackTrace(System.err); 
			System.out.println("<<< Fehler beim Schreiben der Excel-Dateien >>>");
			ergebnis = FEHLER_VERARBEITUNG;		 
		}        
		
		return ergebnis;
		
	}



// Zugehrige Spaltenbeschreibungen einlesen

	public String [][] readClassAndLabels( String id )
	{
		String[][] values = null;
		ArrayList			alFelder  = new ArrayList();
		FileInputStream in = null;
		File f = null;
		String csvDatei = cls_global.getPathMeta()+"/"+id+".csv";
			
			try {

				f = new File( csvDatei );
				byte buffer[] = new byte[ (int) f.length() ];
				in = new FileInputStream( f );
				in.read( buffer, 0, (int) f.length() );

				String str = new String( buffer );

				in.close();

				// parse the data
				values = CSVParser.parse( new StringReader(str),	CSVSEPARATOR );
				
				// min. 1 (data) entry necessary
				if ( values.length <= 1) return null;

				for (int row=1; row<values.length; row++) {
					// no, "VKORG", 1, 3, false, false, "VKOrg"
					alFelder.add(new ColumnDesc ( row+1, values[row][0], Integer.parseInt(values[row][1]), Integer.parseInt(values[row][2]), Boolean.valueOf(values[row][3]).booleanValue(), Boolean.valueOf(values[row][4]).booleanValue(), values[row][5]));
				}
				
				ColumnDesc [] colArr = new ColumnDesc[alFelder.size()];
				//cls_schema.setColDesc( "PLAN_REP", (ColumnDesc []) alFelder.toArray((ColumnDesc[]) colArr) );
				cls_schema.setColDesc( id, (ColumnDesc []) alFelder.toArray((ColumnDesc[]) colArr) );			
			} catch (Exception e) 
			{
				e.printStackTrace(System.err); 
				System.out.println("<<< csv-Datei '"+csvDatei+"' kann nicht gelesen werden :-( >>>");
				return values = null;
			}
		
		System.out.println(csvDatei+" eingelesen");
		
		return values;
	}
	
	
	public boolean readCSV(String csvDatei)
	{
		
		boolean ergebnis=false;
		FileInputStream in = null;
		File f = null;
		try {
			f = new File( csvDatei );
			byte buffer[] = new byte[ (int) f.length() ];
			in = new FileInputStream( f );
			in.read( buffer, 0, (int) f.length() );
			String str = new String( buffer );

			// Parse the data
			String[][] values = ExcelCSVParser.parse(
					new StringReader( str	)	);
			
			in.close();
			
		} catch (Exception e) 
		{
			e.printStackTrace(System.err); 
			System.out.println("<<< csv-Datei '"+csvDatei+"' kann nicht gelesen werden :-( >>>");
			//in.close();
		}
	
	return ergebnis;	
} //  

	
}  // end class



