package mobile;

import dbutils.*;
import mysystem.*;
import java.util.*;
import java.text.*;
import java.io.*;

/** *************************************************************************************
<p>   
  Zusammenstellung globaler Funktionen, die Informationen ber die ISIS-Verzeichnisstruktur
  geben. Die Klasse ist so gestaltet, da die sowohl auf dem Entwicklungs- als auf dem Produktivrechner
  ablauffhig ist.
</p>

*
*  @author   P. Seher
*  @version  1.0 05.2004
*
*/

public class MGlobal {
	
	private String      cls_PfadReports;
	private String      cls_PfadMeta  ;
	private String      cls_PfadSchema  ;
	private String      cls_PfadProgrammeExport;
	private String      cls_PfadDaten ;
	private String      cls_PfadExcelVorlagen;
	private String      cls_PfadExcelBerichte;
	private String      cls_PfadExcelBerichteMonat;
	
	
	public MGlobal()
	{
		cls_PfadReports            = OS.getEnv("HOME")+"/mexreports/REPORTS";
		cls_PfadMeta               = cls_PfadReports+"/META";
		cls_PfadSchema             = cls_PfadMeta+"/schema.ser";
		cls_PfadProgrammeExport    = cls_PfadReports+"/SAS/EXPORT";
		cls_PfadDaten              = cls_PfadReports+"/DATASETS";
		cls_PfadExcelVorlagen      = cls_PfadReports+"/VORLAGEN_EXCEL";
		cls_PfadExcelBerichte      = cls_PfadReports+"/DATASETS/WORK_EXCEL";
		cls_PfadExcelBerichteMonat = cls_PfadReports+"/DATASETS/MONTHLY";
		
		//System.out.println ( "REPORTS:"+cls_PfadReports);
		//System.out.println ( "SCHEMA:"+cls_PfadSchema);
		
	}

	// *********************** getSchema    *****************************************	
	/**
	
	Aktuelles Schema:
	Die serialisierte Schema-Klasse  wird eingelesen und instanziert.
	
	*/   		


	public Schema getSchema()
	{
		Schema ergebnis;
		
		try {
			ergebnis = (Schema) MyIo.readObject(cls_PfadSchema);
			
		} catch (Exception e) 
         	{
			e.printStackTrace(System.err);
			ergebnis = (Schema) null;
		}
		return ergebnis;
		
	}
	// *********************** getPathSchema    *****************************************	
	/**
	
	Absoluter Pfad der serialisierten Schema-Klasse 
	
	*/   		

	public  String getPathSchema()
	{
		return cls_PfadSchema;
	}
	// *********************** getPathProgExport   *****************************************	
	/**
	
	Absoluter Pfad der generierten Export-Programme.
	
	*/   			
	public  String getPathProgExport()
	{
		return cls_PfadProgrammeExport;
	}
	
	
	// *********************** getPathData    *****************************************	
	/**
	
	Absoluter Pfad einer Vertreter-Datendatei
	
	@param  id      Ident der Datei
	@param  persnr  Personalnr. des Vertreters
	
	
	*/   	
	public  String getPathData(String id, String persnr)
	{
		return cls_PfadDaten+"/WORK_DATA_EXPORT/D"+persnr+"/"+id.toLowerCase()+".col";
	}

		
	
	// *********************** getPathTemplate    *****************************************	
	/**
	
	Absoluter Pfad der Vorlage
	
	@param  templateName  Dateiname der Vorlage
	
	
	*/   	
	public  String getPathTemplate(String templateName)
	{
		return cls_PfadExcelVorlagen+"/"+templateName;
	}
	
	

	// *********************** getPathExcelReportsDaily   ***********************************	
	/**
	
	Zielverzeichnis von fr die Excel-Berichte.
	
	
	*/   	
	public  String getPathExcelReportsDaily()
	{
		return cls_PfadExcelBerichte;
	}
	
	// *********************** getPathExcelReportsDaily   ***********************************	
	/**
	
	Zielverzeichnis von fr die Excel-Berichte.
	
	@param  persnr  Personalnr. des Vertreters	
	
	*/   	
	public  String getPathExcelReportsDaily( String persnr)
	{
		return cls_PfadExcelBerichte+"/E"+persnr;
	}		
	
	// *********************** getPathExcelReportDaily    *****************************************	
	/**
	
	Absoluter Pfad einer  Excel-Berichtsdatei bezogen auf einen Vertreter
	
	@param  id      Ident der Datei
	@param  persnr  Personalnr. des Vertreters
	
	
	*/   	
	public  String getPathExcelReportDaily(String excelFileName, String persnr)
	{
		return getPathExcelReportsDaily( persnr)+"/"+excelFileName;
	}
	// *********************** getPathExcelReportsMonthly  ***************** 
	/**
	
	Zielverzeichnis von fr die Excel-Berichte.
	
	
	*/   	
	public  String getPathExcelReportsMonthly()
	{
		return cls_PfadExcelBerichteMonat;
	}
	
	// *********************** getPathExcelReportsMonthly   **************** 
	/**
	
	Zielverzeichnis von fr die Excel-Berichte.
	
	@param  persnr  Personalnr. des Vertreters	
	
	*/   	
	public  String getPathExcelReportMonthly( String persnr)
	{
		return cls_PfadExcelBerichteMonat+"/E"+persnr;
	}		
	
	// *********************** getPathExcelReportsMonthly    *************** 
	/**
	
	Absoluter Pfad einer  Excel-Berichtsdatei bezogen auf einen Vertreter
	
	@param  id      Ident der Datei
	@param  persnr  Personalnr. des Vertreters
	
	
	*/   	
	public  String getPathExcelReportMonthly(String excelFileName, String persnr)
	{
		return getPathExcelReportMonthly( persnr)+"/"+excelFileName;
	}	
	// *********************** getPathSasProperties    ********************* 
	/**
	
	Pfad der SAS-Properties fr der Connect-Aufbau.
	
	*/   	
	public  String getPathSasProperties()
	{
		return cls_PfadReports+"/.MOBILE/connect_sas.properties";
	}
		
	public  String getPathMeta()
	{
		return cls_PfadMeta;
	}
	
	
}

