#! /bin/ksh

# =============================================================================
#
#  PROZEDUR  : datamart_mobile.sh
#
#  ZWECK     : Steuerung der taeglichen MOBILE Datenversorgung 
#
#  AUFRUF    : 
#
#  BEMERKUNG :
#              Wichtige Einzelschritte wurden in Funktionen verpackt,
#              damit diese als solche im Monitoring sichtbar werden
#
#  AUTOR     : P. Seher, Cl. Kraus
#
#  nderungshistorie:
#
# =============================================================================



REPORTS=$HOME/mexreports/REPORTS;  export REPORTS;          # Wurzelverzeichnis fr REPORTS

MOBILEPATH=$REPORTS/DATASETS;   export MOBILEPATH;    # Pfad der Datasets
PERSLIST=$MOBILEPATH/perslist.txt;              # Liste der Personalannummern

# -----------------------------------------------------------------------------
# Initialisierung der STEPSs
# -----------------------------------------------------------------------------

. $REPORTS/setglobal.sh MOBILE
. $REPORTS/setjava.sh


# !!! Wichtig: Monitor-Funktionalitt einschalten (set -m) !!!
set -m
let MODULO=3
let FEHLER=0
FEHLER_TEXT=" ";


# -----------------------------------------------------------------------------
# Funktionen
# -----------------------------------------------------------------------------
# ------------------------------------------------------------------------------ 
# FUNKTION:    ExitProc
# ZWECK:       Verlassen der Prozedur mit Meldung
#
# PARAMETER   :  ExitProc <Exit-Code>  
# BESCHREIBUNG:
# ------------------------------------------------------------------------------
ExitProc()
{

echo ">>> ======================================================================"
echo ">>>  ### $0 beendet `date` ### "
echo ">>>  Startzeit: $START_TIME"
echo ">>>  Stoppzeit: `fzeit`"

let   EXIT_CODE=$1
if [ $EXIT_CODE -ne 0 ]
then
      echo ">>>  Fehlercode: $EXIT_CODE" 
      echo ">>>  Fehlertext: $FEHLER_TEXT"
else 
      echo ">>> Alles O.K. "
fi

echo ">>> ======================================================================"
	
exit $EXIT_CODE	
}


fPreStepDay()
{
  $JAVA mobile.PreStepDay 
  return $?	
}


# ------------------------------------------------------------------------------ 
# FUNKTION:    fCreateVbFiles
# ZWECK:       Erzeugen der Excel-Dateien fuer die Vertreter/VL's
#
# PARAMETER   :  fCreateVbFiles  
# BESCHREIBUNG:  
# ------------------------------------------------------------------------------
fCreateVbFiles()
{

# -----------------------------------------------------------------------------
#  Verarbeitung (i.e. Prozesse) parallelisieren
#  ============================================
#
#  Die Personalnummeren werden in (disjunkte) Restklassen
#  Modulo $MODULO aufgeteilt:
#  Es werden 1 bis $MODULO parallele Prozesse  gestartet, jeder mit einer
#  bestimmen Restklasse als Parameter.
#  Jeder Proze bearbeitet nur die Personalnummern,
#  die seiner Restklasse angehren.
#
# -----------------------------------------------------------------------------

let RC_fCreateVbFiles=0
let REST=0
while [ $REST -lt $MODULO ]
do
  $REPORTS/MOBILE/datamart_mobile_step.sh $MODULO $REST &
  # --- Proze-ID's in einem Array merken
  MPID[$REST]=$!

  let REST=$REST+1
done


# -----------------------------------------------------------------------------
#  Auf das Ende der Verarbeitung warten
# -----------------------------------------------------------------------------

# --- Auf die Prozesse warten

let REST=0
while [ $REST -lt $MODULO ]
do
  ID=${MPID[$REST]}
  echo "Warten auf $ID  ..."
  wait $ID
  if [ $? -ne 0 ]
  then
      let RC_fCreateVbFiles=1
  fi

  let REST=$REST+1
done
 
return $RC_fCreateVbFiles
}


fZipAll()
{
cd $MOBILEPATH/TRANSPORT || { echo "Wechseln ins MOBILPATH-Verzeichnis  $MOBILEPATH   nicht mglich!"; return 21; }

# --- Statistik 'Entwicklung der Dateigroesse einzeln'
for i in *.zip
do
   echo `date +%d.%m.%y` `date +%H:%M:%S` `du -k $i`
done >>$LOGPATH/statistikMobileEinzeln.log

# --- Statistik 'Entwicklung der Dateigroesse gesamt'
cd $MOBILEPATH
echo `date +%d.%m.%y` `date +%H:%M:%S` `du -k TRANSPORT.zip` >>$LOGPATH/statistikMobileGes.log

# --- Gesamtverzeichnis "zippen"
rm -f $MOBILEPATH/TRANSPORT.zip

cd $MOBILEPATH/TRANSPORT || { echo "Wechseln ins MOBILPATH-Verzeichnis  $MOBILEPATH/TRANSPORT    nicht mglich!"; return 21; }
$ZIP -r -o $MOBILEPATH/TRANSPORT.zip * 2>&1 >/dev/null || { echo "Fehler beim ZIPPEN in  $MOBILEPATH/TRANSPORT"; return 99; }

#rm -f $MOBILEPATH/TRANSPORT/*

return 0
}


# ==============================================================================
# VERARBEITUNG starten
# ==============================================================================

# --- Startzeit ausgeben

START_DATE=`fdatum`;
START_TIME=`fzeit`;

echo " ### $0 gestartet `date` ### (Parallelittsgrad: $MODULO) "

# -----------------------------------------------------------------------------
# Lschen vorhandener Mobile-Verzeichnisse und Dateien
# -----------------------------------------------------------------------------

rm -rf $MOBILEPATH/D*
rm -rf $MOBILEPATH/WORK_DATA/D*
rm -rf $MOBILEPATH/TRANSPORT/*
#rm -f   $MOBILEPATH/perslist.txt

echo '$LOGPATH' $LOGPATH


# -----------------------------------------------------------------------------
# An dieser Stelle kann die Personalliste erstellt werden...
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# index.xls einmalig erstellen
# -----------------------------------------------------------------------------
fsh_step fPreStepDay  $LOGPATH/PreStepDay.log 'Mobile PreStepDay'
if [ $? -ne 0 ]
then
	FEHLER_TEXT="Fehler beim Erstellen der Index-Datei"
	ExitProc 1
fi  
	         
# -----------------------------------------------------------------------------
#  Dateien pro VB/VL erstellen
# -----------------------------------------------------------------------------
fsh_step fCreateVbFiles $LOGPATH/create_vb.log 'Mobile CreateVbFiles'
if [ $? -ne 0 ]
then
  FEHLER_TEXT="Erstellen von Dateien fehlerhaft"
  ExitProc 1 
fi

# -----------------------------------------------------------------------------
#  ZIP
# -----------------------------------------------------------------------------
fsh_step fZipAll $LOGPATH/zipall.log 'Mobile ZipAll'
if [ $? -ne 0 ]
then
      FEHLER_TEXT="Zippen fehlerhaft"
      ExitProc 1 
fi


# -----------------------------------------------------------------------------
#  Fertig
# -----------------------------------------------------------------------------
ExitProc 0


