#! /bin/ksh

# =============================================================================
#
#  PROZEDUR  : datamart_mobile_step_persnr.sh
#
#  ZWECK     : Erzeugen der Mobile-Dateien fr eine Personalnr
#
#  AUFRUF    : datamart_mobile_step.sh <PERSNR>  
#
#  BEMERKUNG : Implizit wird ueber die Variable $INCL_MONTH (="TRUE"|"FALSE")
#              gesteuert, ob die Monatsdateien und Doku's
#              mit ausgeliefert werden. 
#
#  AUTOR     : P. Seher, Cl. Kraus
#
# =============================================================================

USAGE="USAGE: $0 <PERSNR> "


# -----------------------------------------------------------------------------
# Prfen der Eingabe
# -----------------------------------------------------------------------------

[ $# = 1 ]||     { echo Ungltige Parameterzahl; echo $USAGE; exit 1; }
PERSNR=$1;                                       export PERSNR;
echo PERSNR:$PERSNR


# -----------------------------------------------------------------------------
# Initialisierung der STEPSs
# -----------------------------------------------------------------------------
REPORTS=$HOME/mexreports/REPORTS;  export REPORTS;          # Wurzelverzeichnis fr REPORTS

MOBILEPATH=$REPORTS/DATASETS;   export MOBILEPATH;    # Pfad der Mobile-Dateien

. $REPORTS/setglobal.sh MOBILE
. $REPORTS/setjava.sh

MIT_MONAT=${MIT_MONAT:-'FALSE'}                  # 'FALSE', falls nicht versorgt


DIR_DATA_ASCII=$MOBILEPATH/WORK_DATA_EXPORT/D$PERSNR; # ASCII-Dateien
DIR_EXCEL=$MOBILEPATH/WORK_EXCEL/E$PERSNR;            # EXCEL-Dateien

# -----------------------------------------------------------------------------
# Erstellen der MOBILE-Dateien pro Persnr.
# -----------------------------------------------------------------------------

#rm -rf  $DIR_DATA_ASCII;
rm -rf  $DIR_EXCEL;


#mkdir  $DIR_DATA_ASCII ||{ echo "Fehler beim Erzeugen des Verzeichnises $DIR_DATA_ASCII"; exit 66; }
mkdir  $DIR_EXCEL ||{ echo "Fehler beim Erzeugen des Verzeichnises $DIR_EXCEL"; exit 66; }


# ---------------------------------
# --- Daten nach EXCEL exportieren
# ---------------------------------
$JAVA mobile.ExportDaily $PERSNR 2>&1   1>>$LOGPATH/L3_$PERSNR.log
if [ $? -ne 0 ]
then
  echo '############### Fehler beim Erstellen der Dateien fr PERSNR:'$PERSNR;
  exit 1
fi  

# ---------------------------------
# --- index.xls kopieren
# ---------------------------------	       
cp $MOBILEPATH/WORK_EXCEL/index.xls $DIR_EXCEL||{ echo "Fehler beim Kopieren von index.xls nach $DIR_EXCEL"; exit 66; }

# ---------------------------------
# --- Monatsdateien und Doku kopieren
# ---------------------------------	       
if [ "X$INCL_MONTH" = "XTRUE" ]
then
    cp $REPORTS/DOKU/* $DIR_EXCEL
    
    cd $DIR_EXCEL||{ echo "Fehler beim Wechseln ins Verzeichnis $DIR_EXCEL"; exit 66; }	 
    
    if test -s $MOBILEPATH/MONTHLY/$PERSNR.zip
       then
			 		unzip -x $MOBILEPATH/MONTHLY/$PERSNR.zip
       fi
fi

# ---------------------------------
# -- ZIP Datei erstellen
# ---------------------------------        
cd $DIR_EXCEL||{ echo "Fehler beim Wechseln ins Verzeichnis $DIR_EXCEL"; exit 66; }
$ZIP -o $MOBILEPATH/TRANSPORT/$PERSNR * 2>&1 >/dev/null || { echo "Fehler beim ZIPPEN bei PERSNR: $PERSNR"; exit 77; }

cd $MOBILEPATH;
# ---------------------------------
# -- Aufraeumen
# ---------------------------------       

#rm -rf  $DIR_DATA_ASCII;
rm -f   $LOGPATH/L*_$PERSNR.log   # - Bei erfolgreichem Lauf, Logging-Datei lschen
rm -rf  $DIR_EXCEL                # - Excel-Dateien loeschen, da schon gezippt

# --- Alles O.K.
exit 0

