#! /bin/ksh

# =============================================================================
#
#  PROZEDUR  : monat_datamart_mobile_d.sh
#
#  ZWECK     : Steuerung der MOBILE Datenversorgung
#
#  AUFRUF    : 
#
#  BEMERKUNG :
#
#  AUTOR     : P. Seher, Cl. Kraus
#
#  nderungshistorie:
#
# =============================================================================



REPORTS=$HOME/mexreports/REPORTS;  export REPORTS;          # Wurzelverzeichnis fr REPORTS

MOBILEPATH=$REPORTS/DATASETS;   export MOBILEPATH;    # Pfad der Datasets
PERSLIST=$MOBILEPATH/perslist.txt;              # Liste der Personalannummern

# -----------------------------------------------------------------------------
# Initialisierung der STEPSs
# -----------------------------------------------------------------------------

. $REPORTS/setglobal.sh MOBILE


# !!! Wichtig: Monitor-Funktionalitt einschalten (set -m) !!!
set -m
let MODULO=3
let FEHLER=0
FEHLER_TEXT=" ";

# -----------------------------------------------------------------------------
# Funktionen
# -----------------------------------------------------------------------------
# ------------------------------------------------------------------------------ 
# FUNKTION:    ExitProc
# ZWECK:       Verlassen der Prozedur mit Meldung
#
# PARAMETER   :  ExitProc <Exit-Code>  
# BESCHREIBUNG:
# ------------------------------------------------------------------------------
ExitProc()
{

echo ">>> ======================================================================"
echo ">>>  ### $0 beendet `date` ### "
echo ">>>  Startzeit: $START_TIME"
echo ">>>  Stoppzeit: `fzeit`"

let   EXIT_CODE=$1
if [ $EXIT_CODE -ne 0 ]
then
      echo ">>>  Fehlercode: $EXIT_CODE" 
      echo ">>>  Fehlertext: $FEHLER_TEXT"
else 
      echo ">>> Alles O.K. "
fi

echo ">>> ======================================================================"
	
exit $EXIT_CODE	
}

# ------------------------------------------------------------------------------ 
# FUNKTION:    fCreateVbFilesM
# ZWECK:       Erzeugen der Excel-Dateien fuer die Vertreter/VL's
#
# PARAMETER   :  fCreateVbFiles  
# BESCHREIBUNG:  
# ------------------------------------------------------------------------------
fCreateVbFilesM()
{
# -----------------------------------------------------------------------------
#  Verarbeitung (i.e. Prozesse) parallelisieren
#  ============================================
#
#  Die Personalnummeren werden in (disjunkte) Restklassen
#  Modulo $MODULO aufgeteilt:
#  Es werden 1 bis $MODULO parallele Prozesse  gestartet, jeder mit einer
#  bestimmen Restklasse als Parameter.
#  Jeder Proze bearbeitet nur die Personalnummern,
#  die seiner Restklasse angehren.
#
# -----------------------------------------------------------------------------

let RC_fCreateVbFilesM=0
let REST=0
while [ $REST -lt $MODULO ]
do
  $REPORTS/MOBILE/monat_datamart_mobile_step.sh $MODULO $REST &
  # --- Proze-ID's in einem Array merken
  MPID[$REST]=$!

  let REST=$REST+1
done


# -----------------------------------------------------------------------------
#  Auf das Ende der Verarbeitung warten
# -----------------------------------------------------------------------------

#sleep 5

# --- Auf die Prozesse warten

let REST=0
while [ $REST -lt $MODULO ]
do
  ID=${MPID[$REST]}
  echo "Warten auf $ID  ..."
  wait $ID
  if [ $? -ne 0 ]
  then
      let RC_fCreateVbFilesM=1
  fi

  let REST=$REST+1
done

return $RC_fCreateVbFilesM

}



# ==============================================================================
# VERARBEITUNG starten
# ==============================================================================

# --- Startzeit ausgeben

START_DATE=`fdatum`;
START_TIME=`fzeit`;

echo " ### $0 gestartet `date` ### (Parallelittsgrad: $MODULO) "

# -----------------------------------------------------------------------------
# Lschen vorhandener Mobile-Verzeichnisse und Dateien
# -----------------------------------------------------------------------------

#rm  -f  $MOBILEPATH/perslist.txt

echo '$LOGPATH' $LOGPATH


# -----------------------------------------------------------------------------
# Personalliste erstellen
# -----------------------------------------------------------------------------


# -----------------------------------------------------------------------------
#  Dateien pro VB/VL erstellen
# -----------------------------------------------------------------------------
 
fsh_step fCreateVbFilesM $LOGPATH/create_vb_monat.log 'Mon-Mobile CreateVbFiles'
if [ $? -ne 0 ]
then
      FEHLER_TEXT="Erstellen von Dateien fehlerhaft"
      ExitProc 1 
fi
# -----------------------------------------------------------------------------
#  Abschlussbehandlung
# -----------------------------------------------------------------------------
END_TIME=`fzeit`;


   
# Ab hier ist alles erfolgreich gelaufen

# --- Statistik 'Entwicklung der Dateigroesse einzeln'
cd $MOBILEPATH/MONTHLY &&
for i in *.zip
do
   echo `date +%d.%m.%y` `date +%H:%M:%S` `du -k $i`
done >>$LOGPATH/statistikMobileMonat.log

# -----------------------------------------------------------------------------
# Eigenschaften des Laufes als Property-Datei schreiben
#  --> wird als Datenstand-Info in Excel weiterverwendet
# -----------------------------------------------------------------------------
GZ=#
while read zeile
do
   echo $zeile
done > $MOBILEPATH/MONTHLY/status.properties <<eeende 
$GZ  
$GZ       Eigenschaften des letzten Monatslaufes
$GZ  
 
START_DATE=$START_DATE
 
START_TIME=$START_TIME
END_TIME=$END_TIME


BM=$BM  
BJ=$BJ
VJ=$VJ
eeende

ExitProc 0


