#! /bin/ksh

# =============================================================================
#
#  PROZEDUR  : monat_datamart_mobile_step.sh
#
#  ZWECK     : Erzeugen der Mobile-Dateien fr eine Untermenge von
#              Personalnummern. Diese Untermenge, gebildet ber Restklassen,
#              wird in Abhngigkeit vom eingestellten Parallelisierungsgrad
#              bestimmt.
#
#  AUFRUF    : monat_datamart_mobile_step.sh <MODULO> <REST>
#
#  BEMERKUNG :
#
#  AUTOR     : P. Seher, Cl. Kraus
#
# =============================================================================



USAGE="USAGE: $0 <MODULO> <REST>"

# -----------------------------------------------------------------------------
# Prfen der Eingabe
# -----------------------------------------------------------------------------

[ $# = 2 ]||     { echo Ungltige Parameterzahl; echo $USAGE; exit 1; }
let MODULO=$1 #|| { echo Ungltige MODULO-Angabe: $1; echo $USAGE ; exit 1; }
let REST=$2   # ||   { echo Ungltige REST-Angabe: $2; echo $USAGE ; exit 1; }

[ $MODULO -gt $REST ]|| { echo MODULO nicht gsser REST; echo $USAGE; exit 1; }
[ $REST -ge  0 ]     || { echo REST nicht positiv; echo $USAGE; exit 1; }


# -----------------------------------------------------------------------------
# Initialisierung der STEPSs
# -----------------------------------------------------------------------------


DATETIME=`date "+%y%m%d.%H%M"`             # Datum und Uhrzeit fr Dateisuffix
REPORTS=$HOME/mexreports/REPORTS;  export REPORTS;          # Wurzelverzeichnis fr REPORTS

MOBILEPATH=$REPORTS/DATASETS/;   export MOBILEPATH;    # Pfad der Datasets
PERSLIST=$MOBILEPATH/perslist.txt
PERSNR=0;                            export PERSNR;


. $REPORTS/setglobal.sh
. $REPORTS/setjava.sh

# -----------------------------------------------------------------------------
# Erstellen der MOBILE-Dateien fr eine Teilmenge von Personalnumern
# -----------------------------------------------------------------------------

let i=0
while read PERSNR
do

 let i=$i+1
 let AREST=$i%$MODULO

 if [ $AREST = $REST ]
 then
      echo "PERSNR:$PERSNR  REST:$REST"
      $REPORTS/MOBILE/monat_datamart_mobile_step_persnr.sh $PERSNR
      if [ $? -ne 0 ]
      then
          echo ">>>! Abbruch wegen Fehler bei PERSNR:$PERSNR"
	  echo ">>>! Prozess $$ mit REST $REST mit Fehler abgebrochen"
	  exit 1
      fi       

 fi

done < $PERSLIST


echo Prozess $$ mit REST $REST erfolgreich beendet
exit 0
